// ExternalFunctionLibrary.cpp: implementation of the CExternalFunctionLibrary class.
//
//////////////////////////////////////////////////////////////////////

/*
   Copyright 2013 Tony Kennedy, Ventana Systems UK

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#include "stdafx.h"
#include "ExternalFunctionLibrary.h"
#include "ExternalFunctionLibraryManager.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CExternalFunctionLibrary::CExternalFunctionLibrary(CExternalFunctionLibraryManager *pManager)
{
	m_pManager = pManager;
	m_version_info_func			= NULL;
	m_funcversion_info_func		= NULL;
	m_set_gv_func				= NULL;
	m_user_definition_func		= NULL;
	m_simulation_setup_func		= NULL;
	m_simulation_shutdown_func	= NULL;
	m_vensim_external_func		= NULL;

	m_hModule = NULL;
	//m_nFunctionCount = 0; // needs to start at 0 so first call hits a valid element of Flist in target lib
	//m_nFirstIndex = -1;
	//m_nLastIndex = -1;
	m_strDLLName = _T("");
	m_nThisLibVersion = -1;
	m_nFuncVersion_Info = -1;
}

CExternalFunctionLibrary::~CExternalFunctionLibrary()
{
	for ( long i = 0 ; i < m_arr_Functions.GetSize() ; i++ )
	{
		CExternalFunctionLibrary_Function *pFunc = m_arr_Functions.GetAt(i);
		delete pFunc;
	}
	
	m_arr_Functions.RemoveAll();

	UnLoadDLL();
}

BOOL CExternalFunctionLibrary::LoadDLL(CString strFilePath, CString strDLLName)
{
	m_hModule = ::LoadLibrary(strFilePath);

	if ( m_hModule == NULL )
		return FALSE;
	
	m_version_info_func = (VERSION_INFO_FUNC)::GetProcAddress(m_hModule,"version_info");
	m_funcversion_info_func = (FUNCVERSION_INFO_FUNC)::GetProcAddress(m_hModule,"funcversion_info");
	m_set_gv_func = (SET_GV_FUNC)::GetProcAddress(m_hModule,"set_gv");
	m_user_definition_func = (USER_DEFINITION_FUNC)::GetProcAddress(m_hModule,"user_definition");
	m_simulation_setup_func = (SIMULATION_SETUP_FUNC)::GetProcAddress(m_hModule,"simulation_setup");
	m_simulation_shutdown_func = (SIMULATION_SHUTDOWN_FUNC)::GetProcAddress(m_hModule,"simulation_shutdown");
	m_vensim_external_func = (VENSIM_EXTERNAL_FUNC)::GetProcAddress(m_hModule,"vensim_external");

	// check that all functions are valid except the simulation shutdown/starup
	if ( m_version_info_func == NULL 
		|| m_funcversion_info_func == NULL 
		//|| m_set_gv_func == NULL 
		|| m_user_definition_func == NULL 
		//|| m_simulation_setup_func == NULL 
		//|| m_simulation_shutdown_func == NULL 
		|| m_vensim_external_func == NULL )
		return FALSE;

	// save name
	m_strDLLPath = strFilePath;
	m_strDLLName = strDLLName;

	//populate the version info.
	_version_info();
	_funcversion_info();
	
	//Load the function information
	LoadFunctionInfo();

	return TRUE;
}

BOOL CExternalFunctionLibrary::UnLoadDLL()
{
	if ( m_hModule != NULL ) {
		FreeLibrary(m_hModule);
	}

	return TRUE;
}

int CExternalFunctionLibrary::_version_info()
{
	m_nThisLibVersion = m_version_info_func();
	return m_nThisLibVersion;
}
int CExternalFunctionLibrary::_funcversion_info()
{
	m_nFuncVersion_Info = m_funcversion_info_func();
	return m_nFuncVersion_Info;
}
int CExternalFunctionLibrary::_set_gv(GLOB_VARS *vgv)
{
	if ( m_set_gv_func != NULL )
		return m_set_gv_func(vgv);

	return TRUE; //can return true here as set_gv is not required.
}

int CExternalFunctionLibrary::UserDefinitionWrapper(int i,unsigned char **sym,unsigned char **argument_desc,int *num_arg,int *num_vector,int *func_index,int *dim_act,int *modify,int *num_loop,int *num_literal,int *num_lookup)
{
	//this returns the values from the function array which have already been checked.
	
	if ( ( i < m_arr_Functions.GetSize() ) && ( i >= 0 ) )
	{
		CExternalFunctionLibrary_Function *pFunction = m_arr_Functions.GetAt(i);
		
		*sym			= pFunction->sym_Vensim ;
		*argument_desc	= pFunction->argument_desc ;
		*num_arg		= pFunction->num_arg ;
		*num_vector		= pFunction->num_vector ;
		*func_index		= pFunction->func_index ;
		*dim_act		= pFunction->dim_act ;
		*modify			= pFunction->modify ;
		*num_loop		= pFunction->num_loop ;
		*num_literal	= pFunction->num_literal ;
		*num_lookup		= pFunction->num_lookup ;

		return 1;
	}

	return 0;
}


int CExternalFunctionLibrary::_user_definition(int i,unsigned char **sym,unsigned char **argument_desc,int *num_arg,int *num_vector,int *func_index,int *dim_act,int *modify,int *num_loop,int *num_literal,int *num_lookup)
{
	int tempint = -1;

	tempint = m_user_definition_func(i,sym,argument_desc,num_arg,num_vector,func_index,dim_act,modify,num_loop,num_literal,num_lookup);

	return tempint;
}

int CExternalFunctionLibrary::FindFunctionID ( int funcid )
{
	for ( int i = 0 ; i < m_arr_Functions.GetSize() ; i++ )
	{
		CExternalFunctionLibrary_Function *pFunc = m_arr_Functions.GetAt(i);

		if ( pFunc->func_index_Vensim  == funcid )
			return pFunc->func_index ;
	}
	return -1;
}

int CExternalFunctionLibrary::_vensim_external(VV *val, int nval, int funcid)
{
	int nExtLibFunctionID = FindFunctionID ( funcid );

	if ( nExtLibFunctionID == -1 )
		return FALSE;

	return m_vensim_external_func(val,nval,nExtLibFunctionID);
}

int CExternalFunctionLibrary::_simulation_setup(int iniflag)
{
	if ( m_simulation_setup_func != NULL )
		return m_simulation_setup_func(iniflag);
	return 0;
}
int CExternalFunctionLibrary::_simulation_shutdown(int finalflag)
{
	if ( m_simulation_shutdown_func != NULL )
		return m_simulation_shutdown_func(finalflag);
	return 0;
}


long CExternalFunctionLibrary::LoadFunctionInfo()
{
	//call UserDefinition until a zero is returned, and store all the info.
	BOOL bBreakLoop = FALSE;
	int nResult;
	long i = 0;

	while ( bBreakLoop == FALSE )
	{
		CExternalFunctionLibrary_Function *pFunction = new CExternalFunctionLibrary_Function();

		nResult = _user_definition(
 					  i //pFunction->i
					, &(pFunction->sym)
					, &(pFunction->argument_desc)
					, &pFunction->num_arg
					, &pFunction->num_vector
					, &pFunction->func_index
					, &pFunction->dim_act
					, &pFunction->modify
					, &pFunction->num_loop
					, &pFunction->num_literal
					, &pFunction->num_lookup );
		
		//pFunction->SetSymVensim();
		
		i++;

		if ( nResult == 0 )
		{
			bBreakLoop = TRUE;
			delete pFunction;
		}
		else
		{
			m_pManager->WriteLogEntry(_T("\tLoaded function : ") + CString(pFunction->sym) );
			m_arr_Functions.Add(pFunction);
		}
	}

	return m_arr_Functions.GetSize();
}
