// ExternalFunctionLibrary.h: interface for the CExternalFunctionLibrary class.
//
//////////////////////////////////////////////////////////////////////

/*
   Copyright 2013 Tony Kennedy, Ventana Systems UK

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#if !defined(AFX_EXTERNALFUNCTIONLIBRARY_H__0F7FE1DF_D038_48F4_BD92_BD2E860E7234__INCLUDED_)
#define AFX_EXTERNALFUNCTIONLIBRARY_H__0F7FE1DF_D038_48F4_BD92_BD2E860E7234__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef int  ( VEFCC /*_export PASCAL*/ *VERSION_INFO_FUNC)();
typedef int  ( VEFCC /*_export PASCAL*/ *FUNCVERSION_INFO_FUNC)();
typedef int  ( VEFCC /*_export PASCAL*/ *SET_GV_FUNC)(GLOB_VARS*);
typedef int  ( VEFCC /*_export PASCAL*/ *USER_DEFINITION_FUNC)(int,unsigned char**,unsigned char**,int*,int*,int*,int*,int*,int*,int*,int*);
typedef int  ( VEFCC /*_export PASCAL*/ *SIMULATION_SETUP_FUNC)(int);
typedef int  ( VEFCC /*_export PASCAL*/ *SIMULATION_SHUTDOWN_FUNC)(int);
typedef int  ( VEFCC /*_export PASCAL*/ *VENSIM_EXTERNAL_FUNC)(VV*,int,int);


#define MAX_STRING_LEN 1000
class CExternalFunctionLibrary_Function : public CObject  
{
public:
	//The following are all returned from user_definition.

	CExternalFunctionLibrary_Function()
	{
		bFunctionNameChanged = FALSE;

		//sym = (unsigned char*)malloc(MAX_STRING_LEN* sizeof(unsigned char));
		//sym_Vensim = (unsigned char*)malloc(MAX_STRING_LEN* sizeof(unsigned char));
		//argument_desc = (unsigned char*)malloc(MAX_STRING_LEN* sizeof(unsigned char));
		
		//sym = new unsigned char[MAX_STRING_LEN];
		sym_Vensim = new unsigned char[MAX_STRING_LEN];
		//argument_desc = new unsigned char[MAX_STRING_LEN];
	}

	virtual ~CExternalFunctionLibrary_Function()
	{
		//delete sym;
		delete sym_Vensim;
		//delete argument_desc;
	}

	int user_definition(unsigned char **symV,unsigned char **argument_descV,int *num_argV,int *num_vectorV,int *func_indexV,int *dim_actV,int *modifyV,int *num_loopV,int *num_literalV,int *num_lookupV)
	{
		if ( bFunctionNameChanged )
			*symV			= this->sym_Vensim ;
		else
			*symV			= this->sym ;

		*argument_descV	= this->argument_desc ;
		*num_argV		= this->num_arg ;
		*num_vectorV	= this->num_vector ;
		*func_indexV	= this->func_index_Vensim ;
		*dim_actV		= this->dim_act ;
		*modifyV		= this->modify ;
		*num_loopV		= this->num_loop ;
		*num_literalV	= this->num_literal ;
		*num_lookupV	= this->num_lookup ;
		
		return 1;
	}
	
	/*
	void SetSymVensim()
	{
		delete [] sym_Vensim;
		sym_Vensim = new unsigned char[strlen((char*)sym)+1];
		//for ( int i = 0 ; i < strlen((char*)sym) ; i++ )
		//sym_Vensim[i] = sym[i];
		sprintf((char*)sym_Vensim,(char*)sym);
	}
	*/

	BOOL bFunctionNameChanged;
	int i;
	unsigned char *sym;
	unsigned char *argument_desc;
	int num_arg;
	int num_vector;
	int func_index;
	int dim_act;
	int modify;
	int num_loop;
	int num_literal;
	int num_lookup;

	//These are the values passed to Vensim.
	long func_index_Vensim;
	unsigned char *sym_Vensim;

	//Some useful functions
	void RenameFunction(CString &strDLLName)
	{
		bFunctionNameChanged = TRUE;
		CStringA strNewFunctionName = strDLLName + _T(" ") + sym;
		delete sym_Vensim;
		sym_Vensim = new unsigned char[strNewFunctionName.GetLength() + 1];
		sprintf( (char*)sym_Vensim , strNewFunctionName.GetBuffer() );
	}

	void SetVensimID(long nFunctionID)
	{
		func_index_Vensim = nFunctionID;
		i = nFunctionID;
	}
};

class CExternalFunctionLibraryManager;

class CExternalFunctionLibrary : public CObject  
{
public:

	CExternalFunctionLibraryManager *m_pManager;
	CExternalFunctionLibrary(CExternalFunctionLibraryManager *pManager);

	virtual ~CExternalFunctionLibrary();
	BOOL LoadDLL(CString strFile, CString strDLLName);
	BOOL UnLoadDLL();
	BOOL IsFunc(long nFunctionID);
	long LoadFunctionInfo();

	CTypedPtrArray<CObArray,CExternalFunctionLibrary_Function*> m_arr_Functions;


private:
	VERSION_INFO_FUNC			m_version_info_func;
	FUNCVERSION_INFO_FUNC		m_funcversion_info_func;
	SET_GV_FUNC					m_set_gv_func;
	USER_DEFINITION_FUNC		m_user_definition_func;
	SIMULATION_SETUP_FUNC		m_simulation_setup_func;
	SIMULATION_SHUTDOWN_FUNC	m_simulation_shutdown_func;
	VENSIM_EXTERNAL_FUNC		m_vensim_external_func;
	
	int m_nThisLibVersion;
	int m_nFuncVersion_Info;
	HINSTANCE m_hModule ;
	//long m_nFunctionCount;
	//long m_nFirstIndex;
	//long m_nLastIndex;
	CString m_strDLLName;
	CString m_strDLLPath;

public:
	CString GetDLLName()
	{
		return m_strDLLName;
	}
	int Get_nFuncVersion_Info()
	{
		return m_nFuncVersion_Info;
	}


	int UserDefinitionWrapper(int i,unsigned char **sym,unsigned char **argument_desc,int *num_arg,int *num_vector,int *func_index,int *dim_act,int *modify,int *num_loop,int *num_literal,int *num_lookup);
	
	int _version_info();
	int _funcversion_info();
	int _set_gv(GLOB_VARS *vgv);
	int _user_definition(int i,unsigned char **sym,unsigned char **argument_desc,int *num_arg,int *num_vector,int *func_index,int *dim_act,int *modify,int *num_loop,int *num_literal,int *num_lookup);
	int _simulation_setup(int iniflag);
	int _simulation_shutdown(int finalflag);
	int _vensim_external(VV *val,int nval, int nFuncID);
	int CExternalFunctionLibrary::FindFunctionID ( int funcid );
	
};

#endif // !defined(AFX_EXTERNALFUNCTIONLIBRARY_H__0F7FE1DF_D038_48F4_BD92_BD2E860E7234__INCLUDED_)
