// MultiExtLib.cpp : Defines the initialization routines for the DLL.
//

/*
   Copyright 2013 Tony Kennedy, Ventana Systems UK

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

/* 
   This library uses HPSLIB for logging:
// Minimal, exportable version of HPSLIB
// Copyright (c) 2007, AlpineSoft http://www.alpinesoft.co.uk
// LGPL applies - i.e. if you use it, please give us a credit
   
   Per the LGPL terms, you may build this project without the HPSLIB
   dependency by including NOLOG in your preprocessor definitions.
   Then the ezutf_src directory may be removed.
*/

#include "stdafx.h"

#include "ExternalFunctionLibrary.h"
#include "ExternalFunctionLibraryManager.h"

#include "MultiExtLib.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//

/////////////////////////////////////////////////////////////////////////////
// CMultiExtLibApp

BEGIN_MESSAGE_MAP(CMultiExtLibApp, CWinApp)
	//{{AFX_MSG_MAP(CMultiExtLibApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMultiExtLibApp construction

CMultiExtLibApp::CMultiExtLibApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMultiExtLibApp object

CMultiExtLibApp theApp;

BOOL CMultiExtLibApp::InitInstance() 
{
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	m_pExtFuncManager = new CExternalFunctionLibraryManager(cmdInfo.m_strFileName, EXTERNAL_VERSION);
	// TODO: Add your specialized code here and/or call the base class

	//read additional DLL's, and load them
	return CWinApp::InitInstance();
}


int CMultiExtLibApp::SetAllGV(GLOB_VARS *vgv)
{
	return m_pExtFuncManager->SetAllGV(vgv);
}
int CMultiExtLibApp::VensimExternal(VV *val,int nval,int funcid)
{
	return m_pExtFuncManager->VensimExternal(val,nval,funcid);
}
int CMultiExtLibApp::SimulationShutdown(int finalflag)
{
	return m_pExtFuncManager->SimulationShutdown(finalflag);
}
int CMultiExtLibApp::SimulationSetup(int iniflag)
{
	return m_pExtFuncManager->SimulationSetup(iniflag);
}
int CMultiExtLibApp::UserDefinition(int i,unsigned char **sym,unsigned char **argument_desc,int *num_arg,int *num_vector,int *func_index,int *dim_act,int *modify,int *num_loop,int *num_literal,int *num_lookup)
{
	return m_pExtFuncManager->UserDefinition(i,sym,argument_desc,num_arg,num_vector,func_index,dim_act,modify,num_loop,num_literal,num_lookup);
}
// Check EXTERNAL_VERSION for all libs
/* no longer needed - checked at load */
int CMultiExtLibApp::CheckAllVersionInfo(int ext_version)
{
	return m_pExtFuncManager->CheckAllVersionInfo(ext_version);
}
// Check funcversion_info for all libs
int CMultiExtLibApp::CheckAllFuncversionInfo(void)
{
	m_pExtFuncManager->CheckAllFuncversionInfo();
	return 1;
}

GLOB_VARS *GV;


CFUNCTION int VEFCC version_info()
{
	/* no longer needed - checked at load time
	theApp.CheckAllVersionInfo(EXTERNAL_VERSION);
	*/
	return(EXTERNAL_VERSION) ;
}

CFUNCTION unsigned short VEFCC funcversion_info()
{
	theApp.CheckAllFuncversionInfo();
	return(1) ;
}

CFUNCTION int VEFCC set_gv(GLOB_VARS *vgv) 
{
	GV = vgv ;

	if(!GV || 
		GV->vgv_magic_start != VGV_MAGIC_START ||
		GV->vgv_magic_end != VGV_MAGIC_END)
		return 0 ;

	if (!theApp.SetAllGV(vgv))
		return 0 ;

	return 1 ;
}

CFUNCTION int VEFCC user_definition(
   int i, /* an index for requesting information - this is mapped to Flist
             but could be used another way - vensim repeatedly calls
             user_definition with i bigger by 1 until user_definition returns
             0 */
   unsigned char **sym,/* the name of the function to be used in the Vensim model */
   unsigned char **argument_desc, /* description of arguments to be shown in equation editor */
   int *num_arg, /* the number of arguments (in Vensim) the function takes 
                    note that for user loop functions this will be one less
                    than the number of arguments the function actually takes on */
   int *num_vector, /* the number of arguments that are passed as real number vectors */
   int *func_index, /* a number between 0 and 32000 that identifies the function
                       vensim_external is called with this number - normally these are sequential */
   int *dim_act,  /* reserved - for doing dimensional analysis but not implemented */
   int *modify,  /* a flag to indicate that the function will modify value that
                    are passed to it - 
                    0 is a normal function that does not modify its argument
                    1 is a function that does modify arguments
                    2 is a function that modifies arguments and serves as a solver
                      of a simultaneous set of conditions as FIND_ZERO */
  int *num_loop, /* the number of loops that are managed by the function - 
                    this is nonzero (normally 1 or 2) for a function that 
                    needs to return a vector or matrix of values - if this is
                    nonzero Vensim will put a pointer to the vector or array to
                    be filled in and pass it as the first argument to the function */

  int *num_literal, /* the number of literals that are passed to the function - 
                       arguments are always passed in the order literals, 
                       lookups, vectors, numbers - if num_loop is set the
                       first argument is a vector even if num_literal is positive */
  int *num_lookup  /* the number of lookup functions passed - this structure is
                      not currently accessible but will be made so in the future */
  )
{
	return theApp.UserDefinition(i,sym,argument_desc,num_arg,num_vector,func_index,dim_act,modify,num_loop,num_literal,num_lookup);
}

CFUNCTION int VEFCC simulation_setup(int iniflag)
{
	return theApp.SimulationSetup(iniflag);
}

CFUNCTION int VEFCC simulation_shutdown(int finalflag)
{
	return theApp.SimulationShutdown(finalflag);
}


CFUNCTION int VEFCC vensim_external(VV *val,int nval,int funcid)
{
	return theApp.VensimExternal(val,nval,funcid);
}


int CMultiExtLibApp::ExitInstance() 
{
	try
	{
		delete m_pExtFuncManager;
	}
	catch(...)
	{
	}
	
	return CWinApp::ExitInstance();
}

